/*
 * globals.c
 *
 * Copyright (c) 2014 Kirill Kryukov
 *
 * This file is a part of the Nucleotide Sequence Clusterizer source code,
 * and may only be used, modified, and distributed under the terms of the
 * license, located in the file LICENSE.txt within this package.
 */

#ifdef INCLUDED_FROM_ALL_C

/*
 * Command line interface: --help and --version flags.
 */
static int opt_help = 0;
static int opt_version = 0;

/*
 * Distance used for clustering.
 */
static char *dist_str = 0;
static unsigned dist = 0;

/*
 * Barcode template, specified with --template option.
 */
static char *template_parameter = 0;
static char *template = 0;

/*
 * Index of the first input file name in the command line.
 */
static int argv_index_of_first_path = -1;

/*
 * Template length.
 */
static int template_length = 0;

/*
 * Specifies which barcode positions are to be used (based on template).
 */
static char *use_barcode_position = 0;




/*
 * Number of meaningful positions in barcode.
 * It's the number of N's in template.
 */
static unsigned barcode_complexity = 0;

/*
 * Number of units and bytes necessary to store one barcode in binary represenatation.
 * With 64-bit units, one unit stores 32 bp of barcode.
 */
static int units_per_barcode = 0;
static int bytes_per_barcode = 0;

/*
 * How many barcodes can fit in one block (for reading).
 */
static int barcodes_per_block = 0;




/*
 * One block of barcodes.
 */
typedef struct block_t
{
    struct block_t *next;
    unit_t units[UNITS_PER_BLOCK];
} block_t;

static block_t *first_block = 0;
static block_t *last_block = 0;
static int n_blocks = 0;
static int n_units_used_in_last_block = 0;
static int n_barcodes_in_last_block = 0;
static int n_barcodes_total = 0;



/*
 * Used to detect nucleotide codes, and to convert them to binary form.
 *    a -> 1
 *    c -> 2
 *    g -> 3
 *    t -> 4
 */
static unsigned is_nucleotide[256];

/*
 * Converts binary representation back to nucleotide character.
 */
static const char binary_to_nucleotide[4] = {'A','C','G','T'};



/*
 * Barcodes stored in volumes for quick sorting and removing duplicates.
 */
static unit_t **index = 0;
static long long *volume_allocated_sizes = 0;
static long long *volume_sizes = 0;



static long long n_unique_barcodes = 0ll;

typedef struct cluster_entry_header_t
{
    void *next;
    long long cluster_id;
} cluster_entry_header_t;
long long cluster_entry_size;
void *cluster_entries = 0;
void **clustered_barcode_index = 0;



/*
 * How many bytes are allocated currently.
 */
static long long allocated = 0ll;

/*
 * Maximum amount of memory that was allocated in current program run.
 */
static long long max_allocated = 0ll;

#endif
